@@ -283,11 +283,12 @@ Handle<JSObject> TypeFeedbackOracle::GetPrototypeForPrimitiveCheck(
     case RECEIVER_MAP_CHECK:
       UNREACHABLE();
       break;
-    case SYMBOL_CHECK:
-      return Handle<JSObject>(native_context_->symbol_delegate());
     case STRING_CHECK:
       function = native_context_->string_function();
       break;
+    case SYMBOL_CHECK:
+      function = native_context_->symbol_function();
+      break;
     case NUMBER_CHECK:
       function = native_context_->number_function();
       break;
