@@ -57,9 +57,9 @@ static void InitializeVM() {
 
 TEST(0) {
   InitializeVM();
-  v8::HandleScope scope;
-
   Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
+
   Assembler assm(isolate, NULL, 0);
 
   __ add(r0, r0, Operand(r1));
@@ -84,9 +84,9 @@ TEST(0) {
 
 TEST(1) {
   InitializeVM();
-  v8::HandleScope scope;
-
   Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
+
   Assembler assm(isolate, NULL, 0);
   Label L, C;
 
@@ -122,9 +122,9 @@ TEST(1) {
 
 TEST(2) {
   InitializeVM();
-  v8::HandleScope scope;
-
   Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
+
   Assembler assm(isolate, NULL, 0);
   Label L, C;
 
@@ -169,7 +169,8 @@ TEST(2) {
 
 TEST(3) {
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   typedef struct {
     int i;
@@ -178,7 +179,6 @@ TEST(3) {
   } T;
   T t;
 
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
   Label L, C;
 
@@ -225,7 +225,8 @@ TEST(3) {
 TEST(4) {
   // Test the VFP floating point instructions.
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   typedef struct {
     double a;
@@ -246,13 +247,12 @@ TEST(4) {
 
   // Create a function that accepts &t, and loads, manipulates, and stores
   // the doubles and floats.
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
   Label L, C;
 
 
   if (CpuFeatures::IsSupported(VFP3)) {
-    CpuFeatures::Scope scope(VFP3);
+    CpuFeatureScope scope(&assm, VFP3);
 
     __ mov(ip, Operand(sp));
     __ stm(db_w, sp, r4.bit() | fp.bit() | lr.bit());
@@ -364,13 +364,13 @@ TEST(4) {
 TEST(5) {
   // Test the ARMv7 bitfield instructions.
   InitializeVM();
-  v8::HandleScope scope;
-
   Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
+
   Assembler assm(isolate, NULL, 0);
 
   if (CpuFeatures::IsSupported(ARMv7)) {
-    CpuFeatures::Scope scope(ARMv7);
+    CpuFeatureScope scope(&assm, ARMv7);
     // On entry, r0 = 0xAAAAAAAA = 0b10..10101010.
     __ ubfx(r0, r0, 1, 12);  // 0b00..010101010101 = 0x555
     __ sbfx(r0, r0, 0, 5);   // 0b11..111111110101 = -11
@@ -401,13 +401,13 @@ TEST(5) {
 TEST(6) {
   // Test saturating instructions.
   InitializeVM();
-  v8::HandleScope scope;
-
   Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
+
   Assembler assm(isolate, NULL, 0);
 
   if (CpuFeatures::IsSupported(ARMv7)) {
-    CpuFeatures::Scope scope(ARMv7);
+    CpuFeatureScope scope(&assm, ARMv7);
     __ usat(r1, 8, Operand(r0));           // Sat 0xFFFF to 0-255 = 0xFF.
     __ usat(r2, 12, Operand(r0, ASR, 9));  // Sat (0xFFFF>>9) to 0-4095 = 0x7F.
     __ usat(r3, 1, Operand(r0, LSL, 16));  // Sat (0xFFFF<<16) to 0-1 = 0x0.
@@ -445,13 +445,13 @@ static void TestRoundingMode(VCVTTypes types,
                              int expected,
                              bool expected_exception = false) {
   InitializeVM();
-  v8::HandleScope scope;
-
   Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
+
   Assembler assm(isolate, NULL, 0);
 
   if (CpuFeatures::IsSupported(VFP3)) {
-    CpuFeatures::Scope scope(VFP3);
+    CpuFeatureScope scope(&assm, VFP3);
 
     Label wrong_exception;
 
@@ -623,7 +623,8 @@ TEST(7) {
 TEST(8) {
   // Test VFP multi load/store with ia_w.
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   typedef struct {
     double a;
@@ -651,11 +652,10 @@ TEST(8) {
 
   // Create a function that uses vldm/vstm to move some double and
   // single precision values around in memory.
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
 
   if (CpuFeatures::IsSupported(VFP2)) {
-    CpuFeatures::Scope scope(VFP2);
+    CpuFeatureScope scope(&assm, VFP2);
 
     __ mov(ip, Operand(sp));
     __ stm(db_w, sp, r4.bit() | fp.bit() | lr.bit());
@@ -735,7 +735,8 @@ TEST(8) {
 TEST(9) {
   // Test VFP multi load/store with ia.
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   typedef struct {
     double a;
@@ -763,11 +764,10 @@ TEST(9) {
 
   // Create a function that uses vldm/vstm to move some double and
   // single precision values around in memory.
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
 
   if (CpuFeatures::IsSupported(VFP2)) {
-    CpuFeatures::Scope scope(VFP2);
+    CpuFeatureScope scope(&assm, VFP2);
 
     __ mov(ip, Operand(sp));
     __ stm(db_w, sp, r4.bit() | fp.bit() | lr.bit());
@@ -851,7 +851,8 @@ TEST(9) {
 TEST(10) {
   // Test VFP multi load/store with db_w.
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   typedef struct {
     double a;
@@ -879,11 +880,10 @@ TEST(10) {
 
   // Create a function that uses vldm/vstm to move some double and
   // single precision values around in memory.
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
 
   if (CpuFeatures::IsSupported(VFP2)) {
-    CpuFeatures::Scope scope(VFP2);
+    CpuFeatureScope scope(&assm, VFP2);
 
     __ mov(ip, Operand(sp));
     __ stm(db_w, sp, r4.bit() | fp.bit() | lr.bit());
@@ -963,7 +963,8 @@ TEST(10) {
 TEST(11) {
   // Test instructions using the carry flag.
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   typedef struct {
     int32_t a;
@@ -976,7 +977,6 @@ TEST(11) {
   i.a = 0xabcd0001;
   i.b = 0xabcd0000;
 
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
 
   // Test HeapObject untagging.
@@ -1029,9 +1029,10 @@ TEST(11) {
 TEST(12) {
   // Test chaining of label usages within instructions (issue 1644).
   InitializeVM();
-  v8::HandleScope scope;
-  Assembler assm(Isolate::Current(), NULL, 0);
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
+  Assembler assm(isolate, NULL, 0);
   Label target;
   __ b(eq, &target);
   __ b(ne, &target);
@@ -1043,7 +1044,8 @@ TEST(12) {
 TEST(13) {
   // Test VFP instructions using registers d16-d31.
   InitializeVM();
-  v8::HandleScope scope;
+  Isolate* isolate = Isolate::Current();
+  HandleScope scope(isolate);
 
   if (!CpuFeatures::IsSupported(VFP32DREGS)) {
     return;
@@ -1064,13 +1066,12 @@ TEST(13) {
 
   // Create a function that accepts &t, and loads, manipulates, and stores
   // the doubles and floats.
-  Isolate* isolate = Isolate::Current();
   Assembler assm(isolate, NULL, 0);
   Label L, C;
 
 
   if (CpuFeatures::IsSupported(VFP3)) {
-    CpuFeatures::Scope scope(VFP3);
+    CpuFeatureScope scope(&assm, VFP3);
 
     __ stm(db_w, sp, r4.bit() | lr.bit());
 
